/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus header file for type conversion functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _BRUTUS_CONV_H_
#define _BRUTUS_CONV_H_

#include <glib.h>
#include <time.h>
#include <idl_output/types.h>
#include <idl_output/defines.h>

G_BEGIN_DECLS

#ifdef UTC_TIME_FORMAT_STRING
#undef UTC_TIME_FORMAT_STRING
#endif
#define UTC_TIME_FORMAT_STRING "%Y-%m-%dT%H:%M:%SZ"

#define BRUTUS_PROP_TYPE(__tag__) ((guint16)(__tag__ & BRUTUS_BRUTUS_PROP_TYPE_MASK))

#define BRUTUS_PROP_ID(__tag__)  ((guint16)(__tag__ >> 16))

struct BrutusGregorianTime {
	gint bt_year;      /* Gregorian year : [G_MININT32,0[ U ]0,G_MAXINT32] */
	guint bt_month;    /* [1,12] */
	guint bt_day;      /* [1,31] */
	guint bt_hour;     /* [0,23] */
	guint bt_minutes;  /* [0,59] */
	guint bt_seconds;  /* [0,60] - only 60 for leap seconds, which (BTW) is ignored */
	guint bt_nanosecs; /* [0,999999999] */
};


/*
 * Converts a WIN32 FILETIME into a UNIX time_t. The FILETIME must
 * refer to a time on or after UNIX epoch (00:00:00 at January 1. 1970).
 * The return value is rounded towards zero to the nearest second.
 */
extern time_t brutus_filetime_to_time_t(const BRUTUS_FILETIME ft);

/*
 * Converts a UNIX time_t into a WIN32 FILETIME.
 */
extern void brutus_time_t_to_filetime(const time_t tt,
				      BRUTUS_FILETIME *ft);

/*
 * Converts 'struct BrutusGregorianTime' into a WIN32 FILETIME.
 */
extern void brutus_bgt_to_filetime(const struct BrutusGregorianTime *bgt,
				   BRUTUS_FILETIME *ft);


/*
 * Converts WIN32 FILETIME into a 'struct BrutusGregorianTime'.
 */
extern void brutus_filetime_to_bgt(const BRUTUS_FILETIME *ft,
				   struct BrutusGregorianTime *bgt);

/*
 * Converts a BRUTUS_BRESULT into a WIN32 HRESULT as a string.
 */
extern const gchar *brutus_bresult_to_str(const BRUTUS_BRESULT br);

/*
 * Converts a string defined by 'format' into a time_t.
 */
extern void brutus_string_to_time_t(const char *str,
				    const char *format,
				    time_t *timep);
/*
 * Converts a time_t into a string defined by 'format'.
 */
extern void brutus_time_t_to_string(const time_t timep,
				    const size_t len,
				    const char *format,
				    char *str);

/*
 * Returns the property tag as a string.
 */
extern const gchar *brutus_proptag_to_str(const BRUTUS_BDEFINE prop_tag);

/*
 * Returns the property type as a string.
 */
extern const gchar *brutus_proptype_to_str(const BRUTUS_PT_TYPE prop_type);

G_END_DECLS

#endif // _BRUTUS_CONV_H_
