/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Servant implementation file for a Brutus proxy daemon.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#include <errno.h>
#include <session/servants/BrutusCheck.h>

#ifdef IGNORE_RETURN_VALUE
#undef IGNORE_RETURN_VALUE
#endif
#define IGNORE_RETURN_VALUE(__x__) G_STMT_START { if (__x__) ; } G_STMT_END

#ifndef ORBIT2_EX
#define ORBIT2_EX(_ev_) ((NULL != _ev_) && ((_ev_)->_major != CORBA_NO_EXCEPTION))
#endif

/*** App-specific servant structures ***/

#if !defined(_typedef_impl_POA_BRUTUS_BrutusCheck_)
#define _typedef_impl_POA_BRUTUS_BrutusCheck_ 1
typedef struct {
	POA_BRUTUS_BrutusCheck servant;
	PortableServer_POA poa;

	/* ------ add private attributes here ------ */
	BRUTUS_BrutusCheck filling;
	/* ------ ---------- end ------------ ------ */

} impl_POA_BRUTUS_BrutusCheck;
#endif


/*** Implementation stub prototypes ***/

#if !defined(_decl_impl_BRUTUS_BrutusCheck__destroy_)
#define _decl_impl_BRUTUS_BrutusCheck__destroy_ 1
static void impl_BRUTUS_BrutusCheck__destroy(impl_POA_BRUTUS_BrutusCheck *servant,
					     CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusCheck_ping_)
#define _decl_impl_BRUTUS_BrutusCheck_ping_ 1
static void
impl_BRUTUS_BrutusCheck_ping(impl_POA_BRUTUS_BrutusCheck *servant,
			     CORBA_Environment *ev);
#endif


/*** epv structures ***/

#if !defined(_impl_BRUTUS_BrutusCheck_base_epv_)
#define _impl_BRUTUS_BrutusCheck_base_epv_ 1
static PortableServer_ServantBase__epv impl_BRUTUS_BrutusCheck_base_epv = {
	NULL,             /* _private data */
	(gpointer) & impl_BRUTUS_BrutusCheck__destroy, /* finalize routine */
	NULL,             /* default_POA routine */
};
#endif

#if !defined(_impl_BRUTUS_BrutusCheck_epv_)
#define _impl_BRUTUS_BrutusCheck_epv_ 1
static POA_BRUTUS_BrutusCheck__epv impl_BRUTUS_BrutusCheck_epv = {
	NULL, /* _private */
	(gpointer)&impl_BRUTUS_BrutusCheck_ping,
};
#endif


/*** vepv structures ***/

#if !defined(_impl_BRUTUS_BrutusCheck_vepv_)
#define _impl_BRUTUS_BrutusCheck_vepv_ 1
static POA_BRUTUS_BrutusCheck__vepv impl_BRUTUS_BrutusCheck_vepv = {
	&impl_BRUTUS_BrutusCheck_base_epv,
	&impl_BRUTUS_BrutusCheck_epv,
};
#endif


/*** Stub implementations ***/

#if !defined(_impl_BRUTUS_BrutusCheck__create_)
#define _impl_BRUTUS_BrutusCheck__create_ 1
static BRUTUS_BrutusCheck impl_BRUTUS_BrutusCheck__create(BRUTUS_BrutusCheck content,
							  PortableServer_POA poa,
							  CORBA_Environment *ev)
{
	BRUTUS_BrutusCheck retval = CORBA_OBJECT_NIL;
	impl_POA_BRUTUS_BrutusCheck *newservant = NULL;
	PortableServer_ObjectId *objid = NULL;

	newservant = g_new0(impl_POA_BRUTUS_BrutusCheck, 1);
	if (!newservant) {
		CORBA_exception_set_system(ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
		return CORBA_OBJECT_NIL;
	}
	newservant->servant.vepv = &impl_BRUTUS_BrutusCheck_vepv;
	newservant->poa = (PortableServer_POA) CORBA_Object_duplicate((CORBA_Object)poa, ev);

	POA_BRUTUS_BrutusCheck__init((PortableServer_Servant)newservant, ev);

	/* ------ init private attributes here ------ */
	newservant->filling = (CORBA_OBJECT_NIL == content) ? CORBA_OBJECT_NIL : CORBA_Object_duplicate((CORBA_Object)content, ev);
	/* ------ ---------- end ------------- ------ */

	objid = PortableServer_POA_activate_object(poa, newservant, ev);
	if (!objid) {
		if (CORBA_OBJECT_NIL != newservant->filling)
			CORBA_Object_release(newservant->filling, ev);
		g_free(newservant);
		return CORBA_OBJECT_NIL;
	}
	CORBA_free(objid);
	retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

	return retval;
}
#endif

#if !defined(_impl_BRUTUS_BrutusCheck__destroy_)
#define _impl_BRUTUS_BrutusCheck__destroy_ 1
static void
impl_BRUTUS_BrutusCheck__destroy(impl_POA_BRUTUS_BrutusCheck *servant,
				 CORBA_Environment *ev)
{
	CORBA_Object_release ((CORBA_Object) servant->poa, ev);

	/* No further remote method calls are delegated to
	 * servant and you may free your private attributes. */

	/* ------ free private attributes here ------ */

	if (CORBA_OBJECT_NIL != servant->filling)
		CORBA_Object_release(servant->filling, ev);

	/* ------ ---------- end ------------- ------ */

	POA_BRUTUS_BrutusCheck__fini((PortableServer_Servant)servant, ev);

	g_free(servant);
}
#endif

/*
 * Will return the process ID of executable or 0 (zero) in error.
 *
 * Usefull configure.in define:
 *
 * EVO_PATHNAME : Full path to Evolution executable
 */
static inline pid_t
brutus_get_process_pid(const char *executable)
{
	pid_t retv = 0;
	DIR *top = NULL;
	DIR *child = NULL;
	struct dirent *ep = NULL;
	struct dirent *child_ep = NULL;
	struct stat dir_stat;
	const uid_t uid = getuid();
	char buf[512] = { '\0' };
	gboolean found = FALSE;

	top = opendir("/proc");
	if (!top)
		return 0;

	if (chdir("/proc"))
		goto out;

	// scanning /proc
	while ((ep = readdir(top))) {
		if (('\0' == ep->d_name[0]) || !strcmp(".", ep->d_name) || !strcmp("..", ep->d_name))
			continue;

		if (stat(ep->d_name, &dir_stat))
			continue;

		// is it our process?
		if (uid != dir_stat.st_uid)
			continue;

		// is it a directory?
		child = opendir(ep->d_name);
		if (!child)
			continue;
		if (!chdir(ep->d_name)) {
			// scanning /proc/<child>
			while ((child_ep = readdir(child))) {
				if (strcmp("exe", (child_ep->d_name)))
					continue;

				memset((void*)buf, '\0', sizeof(buf));
				if (-1 != readlink(child_ep->d_name, buf, sizeof(buf))) {
					if (!strcmp(executable, buf)) {
						found = TRUE;
						retv = (pid_t)atoi(ep->d_name);
						break;
					}
				}
			}
			if (chdir("/proc")) {
				IGNORE_RETURN_VALUE(closedir(child));
				break;
			}
		}
		IGNORE_RETURN_VALUE(closedir(child));
		if (found)
			break;
	}

out:
	IGNORE_RETURN_VALUE(closedir(top));

	return retv;
}

#if !defined(_impl_BRUTUS_BrutusCheck_ping_)
#define _impl_BRUTUS_BrutusCheck_ping_ 1
static void
impl_BRUTUS_BrutusCheck_ping(impl_POA_BRUTUS_BrutusCheck *servant,
			     CORBA_Environment *ev)
{
	// do not answer if the Evolution shell is down
	if (!brutus_get_process_pid(EVO_PATHNAME)) {
		if (CORBA_OBJECT_NIL != servant->filling) {
			CORBA_Object_release(servant->filling, ev);
			servant->filling = CORBA_OBJECT_NIL;
		}
		CORBA_exception_set_system(ev, ex_CORBA_OBJECT_NOT_EXIST, CORBA_COMPLETED_NO);
		return;
	}
	
	if (CORBA_OBJECT_NIL == servant->filling) {
		CORBA_exception_set_system(ev, ex_CORBA_OBJECT_NOT_EXIST, CORBA_COMPLETED_NO);
		return;
	}

	BRUTUS_BrutusCheck_ping(servant->filling, ev);
	if (ORBIT2_EX(ev)) {
		CORBA_Object_release(servant->filling, ev);
		servant->filling = CORBA_OBJECT_NIL;
		CORBA_exception_set_system(ev, ex_CORBA_OBJECT_NOT_EXIST, CORBA_COMPLETED_NO);
	}

	return;
}
#endif
